## CASAplotms 

CASA plotms has been built and distributed as part of [CASA](http://casa.nrao.edu/) for many years. This repository makes it possible to build the CASA plotms without building the rest of CASA. At some point in the near future, CASAplotms (built from this repository) will become the default plotms application for CASA distributions.

## Check Out

You should checkout CASAplotms with:
```
-bash-4.2$ git clone --recursive https://open-bitbucket.nrao.edu/scm/casa/CASAplotms.git
```

## Building

PlotMS required Qt 4.8 or later. Check which version of Qt is available using ```which```:
```
-bash-4.2$ which qmake
/usr/lib64/qt-3.3/bin/qmake
-bash-4.2$ PATH=/usr/lib64/qt4/bin:$PATH
-bash-4.2$ which qmake
/usr/lib64/qt4/bin/qmake
-bash-4.2$ 
```
In addition, the ```casatools``` Python module is required for building and the ```casatasks``` Python module is required for starting PlotMS from the python prompt. You can check by attempting to get help from the ```casatools``` module:
```
-bash-4.2$ python3 -m casatools --help
/opt/rh/rh-python36/root/usr/bin/python3: No module named casatools
-bash-4.2$ PYTHONPATH=../casatools/build/lib.linux-x86_64-3.6 python3 -m casatools --help
--compiler-cc       path to C compiler used to build casatools
--compiler-cxx      path to C++ compiler used to build casatools
--compiler-xml      path to compiler used to generate bindings from XML
--grpc-compile      flags to compile C++ source files
--grpc-link         flags to use to link gRPC C++ applications
--grpc-libpath      path to gRPC C++ libraries
--grpc-protocpp     path to C++ protoc compiler
--grpc-protopy      path to Python protoc compiler
--proto-registrar   path to registrar protobuf spec
--proto-shutdown    path to shutdown protobuf spec
--proto-ping        path to ping protobuf spec
-bash-4.2$ 
```
If you had to set ```PYTHONPATH``` environment variable, you need to also set it similarily when running qmake in the following You should be able to build CASA plotms on both OSX and Linux with:
```
-bash-4.2$ PYTHONPATH=../casatools/build/lib.linux-x86_64-3.6 qmake
-bash-4.2$ make
```
This process should result in the creation of an [AppImage](https://appimage.org/) which can be run directly:
```
-bash-4.2$ ls build/bin.3.10.0-1062.el7.x86_64/casaplotms-x86_64.AppImage 
build/bin.3.10.0-1062.el7.x86_64/casaplotms-x86_64.AppImage
-bash-4.2$ build/bin.3.10.0-1062.el7.x86_64/casaplotms-x86_64.AppImage
2019-10-02 19:10:02	INFO	PlotMS::params_changed	Updating: REDRAW, MSDATA, CACHE, AXES, CANVAS, DISPLAY, PAGEHEADER, ITERATION, LOG, PLOTMS_OPTIONS.
-bash-4.2$ 
```
This AppImage could be distributed for use as is by just copying it to any desired location. It is also possible to build a Python binding for plotms. The ```casatools``` module is also required for this step:
```
-bash-4.2$ PYTHONPATH=../casatools/build/lib.linux-x86_64-3.6 make py
```
This should result in a Python module which could be added to ```PYTHONPATH``` directly and a [PyPI](https://pypi.org/) wheel that can be installed with [pip](https://pypi.org/project/pip/):
```
-bash-4.2$ ls build/python.3.10.0-1062.el7.x86_64/casaplotms
__bin__  gotasks  __init__.py  plotms.py  private  __pycache__
-bash-4.2$ ls build/python.3.10.0-1062.el7.x86_64/dist
casaplotms-2019.275-py3-none-linux_x86_64.whl
-bash-4.2$ 
```
If the ```casaplotms``` wheel is installed, it should install its dependencies. These include:
*  casatools -- CASA tools and configuration
*  casatasks -- CASA tasks
*  grpcio -- [gRPC](https://grpc.io/) binding for Python
*  protobuf -- transport layer for gRPC

If the ```casaplotms``` is loaded directly, all of these modules must be available in your ```PYTHONPATH```. If you have the CASA development environment, these packages should be installed with as part of the ```casa-toolset-3``` RPM.

## Caveats

CASA plotms requires version 2.0 or greater of the GNU Scientific Library (GSL). If
you have an older version of GSL, you will get errors like:
```
.../FringeJones.cc:1501: undefined reference to `gsl_multilarge_nlinear_trust'
```
RedHat Enterprise Linux 7 (RHEL7) for example, comes with version 1.15. ```qmake``` is very
insistent on the order that link paths are added to the link line. As a result, it
may be necessary to uninstall the development files for the older version of GSL.
On RHEL7, this was necessary. The command to do this on RHEL7 (as root) is:
```
-bash-4.2# yum remove gsl-devel
```
